from ctypes import *
import sys
from dwfconstants import *
import serial

# 仮想COMポートの定義
PORT = "COM3"
BAUD_RATE = 115200

# WaveForms SDKのランタイムをロードする。
dwf = cdll.dwf
hdwf = c_int()
channel = c_int(0)

# AD2に接続する。
dwf.FDwfParamSet(DwfParamOnClose, c_int(1))
dwf.FDwfDeviceOpen(c_int(-1), byref(hdwf))
if hdwf.value == hdwfNone.value:
    print("failed to open device")
    quit()
dwf.FDwfDeviceAutoConfigureSet(hdwf, c_int(0))

# 仮想COMポートを開く。
ser = serial.Serial(PORT, BAUD_RATE)
wstable = {b'0': funcSine, b'1': funcSquare, b'2': funcRampUp}
try:
    while True:
        # 仮想COMポートから文字列を1行分読み込む。
        param = ser.readline().split()
        # 読み込んだ文字列をパラメータのリストに分解する。
        wave_out = (param[0] == b'1')
        wave_shape = wstable.get(param[1], funcSine)
        frequency = int(param[2])
        amplitude = float(param[3])
        # 音声出力のON/OFFを切り替える
        if wave_out:
            dwf.FDwfAnalogOutNodeEnableSet(hdwf, channel, AnalogOutNodeCarrier, c_int(1))
        else:
            dwf.FDwfAnalogOutNodeEnableSet(hdwf, channel, AnalogOutNodeCarrier, c_int(0))

        # 与えられた波形、周波数、振幅のアナログは系を出力する。
        dwf.FDwfAnalogOutNodeFunctionSet(hdwf, channel, AnalogOutNodeCarrier, wave_shape)
        dwf.FDwfAnalogOutNodeFrequencySet(hdwf, channel, AnalogOutNodeCarrier, c_double(frequency))
        dwf.FDwfAnalogOutNodeAmplitudeSet(hdwf, channel, AnalogOutNodeCarrier, c_double(amplitude))
        dwf.FDwfAnalogOutConfigure(hdwf, channel, c_int(1))
        print(wave_shape, frequency, amplitude)
except KeyboardInterrupt:
    # キーボード割り込みが入ったら停止する。
    pass

# AD２との接続を解放する。
dwf.FDwfAnalogOutReset(hdwf, c_int(-1))
dwf.FDwfDeviceCloseAll(hdwf)
